package vintageLocos;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

public class TrainStation {

    private final List<Locomotive> locomotives;
    private final String name;
    private final int capacity;
    private final int railGauge;

    public TrainStation(String name, int capacity, int railGauge) {
        this.locomotives = new ArrayList<>();
        this.name = name;
        this.capacity = capacity;
        this.railGauge = railGauge;
    }

    public void addLocomotive(Locomotive locomotive) {

        int difference = Math.abs(locomotive.getGauge() - this.railGauge);

        if (locomotives.size() < capacity) {
            if (this.getRailGauge() == locomotive.getGauge()) {
                if (locomotives.stream().anyMatch(s -> s.getName().equals(locomotive.getName()))) {
                    throw new RuntimeException("This locomotive already exists!");
                }
                locomotives.add(locomotive);
            } else {
                System.out.printf("The rail gauge of this station does not match the locomotive gauge!"
                        + " Difference: %d mm.\n", difference);
            }
        } else {
            System.out.println(("This train station is full!"));
        }
    }

    public Locomotive getLocomotive(String name) {
        return locomotives.stream().filter(f -> f.getName().equals(name)).findFirst().orElse(null);
    }

    public boolean removeLocomotive(String name) {
        Locomotive locomotive = this.getLocomotive(name);
        locomotives.remove(locomotive);
        return locomotive != null;
    }

    public int getCount() {
        return locomotives.size();
    }

    public String getFastestLocomotive() {
        Locomotive fastestLocomotive = locomotives.stream().max(Comparator.comparing(Locomotive::getMaxSpeed)).orElse(null);
        if(fastestLocomotive != null) {
            return String.format(fastestLocomotive.getName() + " is the fastest locomotive with a maximum speed of " + fastestLocomotive.getMaxSpeed() + " km/h.");
        }
        return "There are no locomotives.";
    }

    public String getOldestLocomotive() {
        Locomotive oldestLocomotive = locomotives.stream().min(Comparator.comparing(Locomotive::getBuildDate)).orElse(null);
        if(oldestLocomotive != null) {
        return oldestLocomotive.getName();
        }
        return "There are no locomotives.";
    }

    public List<Locomotive> getLocomotives() {
        return locomotives;
    }

    public String getName() {
        return name;
    }

    public int getCapacity() {
        return capacity;
    }

    public int getRailGauge() {
        return railGauge;
    }

    public String getStatistics() {

        TrainStation trainStation = new TrainStation(getName(), getCapacity(), railGauge);
        StringBuilder build = new StringBuilder();

        if (locomotives.isEmpty()) {
            build.append(System.lineSeparator()).append("There are no locomotives departing from ").append(trainStation.getName()).append(" station.");
        } else {
            build.append("Locomotives departed from ").append(trainStation.getName()).append(":");
            build.append(System.lineSeparator());
            for (int i = 0; i < locomotives.size(); i++) {
                build.append(i + 1).append(". ").append(locomotives.get(i).getName()).append(System.lineSeparator());
            }
        }
//        locomotives.forEach(locomotive -> build.append(locomotive.getName()).append(System.lineSeparator()));
        return build.toString().trim();
    }
}
